/************************************************************************
	Copyright (C) 2010 Robert Scott.
	
	This file if part of envi_feed.
	
    envi_feed is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    envi_feed is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
	
*************************************************************************/

#include "config.h"
#include <exception>
#include <string>
#include <boost/scoped_array.hpp>
#include <json/value.h>
#include <json/writer.h>
#include <tinyxml.h>

#include "envi.h"



bool envi::data::parse()
{
	//An envi display unit is what we call a logger
	//An envi sensor is what we call a monitor
	//An envi channel is what we call a sensor (attached to a monitor channel)
	std::size_t pos = xml_.find("<msg>");
	if(pos != 0) {
		error_="Missing <msg> at start";
		return false;
	}	
	pos = xml_.rfind("</msg>");
	if(pos != xml_.length()-6) {
		error_="Missing </msg> at end";
		return false;
	}
	
	TiXmlDocument doc;
	boost::scoped_array<char> xml_s(new char[xml_.length()+1]);
	strcpy(xml_s.get(), xml_.c_str());
	doc.Parse(xml_s.get());
	TiXmlNode *root = doc.FirstChild("msg");
	if(!root) {
		error_ = "Missing root node";
		return false;
	}


	TiXmlNode *node = root->FirstChild("tmpr");
	if(!node) {
		error_ = "Missing tmpr node";
		return false;
	}
	boost::shared_ptr<monitor> t_monitor(new monitor("MT",-1)); 
	boost::shared_ptr<sensor> t_sensor(new sensor(1, "temperature",node->FirstChild()->Value())); 
	t_monitor->getSensors().push_back(t_sensor);
	monitors_.push_back(t_monitor);
	
	node = root->FirstChild("sensor"); 
	if(!node) {
		error_ = "Missing sensor node";
		return false;
	}
	std::ostringstream stream;
	stream << "M" << node->FirstChild()->Value();
	boost::shared_ptr<monitor> p_monitor(new monitor(stream.str(), atoi(node->FirstChild()->Value())));  

	boost::shared_ptr<sensor> s1(new sensor(1,"power"));	
	node = root->FirstChild("ch1");
	if(node) {
		node = node->FirstChild("watts");
		if(node) {
			s1->setValue(node->FirstChild()->Value());
		}
	}
	p_monitor->getSensors().push_back(s1);

	boost::shared_ptr<sensor> s2(new sensor(2,"power"));
	node = root->FirstChild("ch2");
	if(node) {
		node = node->FirstChild("watts");
		if(node) {
			s2->setValue(node->FirstChild()->Value());
		}
	}
	p_monitor->getSensors().push_back(s2);

	boost::shared_ptr<sensor> s3(new sensor(3,"power"));
	node = root->FirstChild("ch3");
	if(node) {
		node = node->FirstChild("watts");
		if(node) {
			s3->setValue(node->FirstChild()->Value());
		}
	}
	p_monitor->getSensors().push_back(s3);	
	monitors_.push_back(p_monitor);

	
	valid_=true;
	return true;
}

std::string envi::data::to_json()
{
	Json::Value root;
	Json::FastWriter writer;
	std::ostringstream stream;
	root["logger"] = logger_code_;
	stream.str("");
	stream << time_;
	root["time"] = stream.str();
	int index1=0;
	for(std::vector<boost::shared_ptr< monitor> >::const_iterator itr1 = monitors_.begin(); itr1!=monitors_.end(); ++itr1) {
		root["monitor"][index1]["code"] = (*itr1)->getCode();
		root["monitor"][index1]["channel"] = (*itr1)->getLoggerChannelNumber();
		int index2=0;
		std::vector<boost::shared_ptr<sensor> > sensors = (*itr1)->getSensors();
		for(std::vector<boost::shared_ptr<sensor> >::const_iterator itr2=sensors.begin(); itr2!=sensors.end(); ++itr2) {
			root["monitor"][index1]["sensor"][index2]["channel"] = (*itr2)->getChannelNumber();
			root["monitor"][index1]["sensor"][index2]["type"] = (*itr2)->getType();
			root["monitor"][index1]["sensor"][index2]["value"] = (*itr2)->getValue();
			++index2;
		} 
		++index1;
	}
	
	std::string json=writer.write(root);
	return json;
}