/************************************************************************
	Copyright (C) 2010 Robert Scott.
	
	This file if part of envi_feed.
	
    envi_feed is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    envi_feed is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
	
*************************************************************************/

#ifndef device_h__
#define device_h__

#include <deque>
#include <map>
#include <boost/shared_ptr.hpp>
#include <boost/asio.hpp>
#include <boost/asio/serial_port.hpp>
#include <boost/thread.hpp>
#include <ctime>
#include <zmq.hpp>



class TiXmlElement;

class sensor {
private:
	int channel_number_;
	std::string type_;
	std::string value_;
public:
	//Constructor
	sensor():channel_number_(0), type_(""), value_("U") {}
	sensor(int channel_number, const std::string & type): channel_number_(channel_number), type_(type), value_("U") {}
	sensor(int channel_number, const std::string & type, const std::string & value):channel_number_(channel_number), type_(type), value_(value) {}
	//Setters
	void setChannelNumber(int value) { channel_number_=value; }
	void setType(const std::string & value) { type_=value; }
	void setValue(const std::string & value) { value_=value; }
	
	//Getters
	int getChannelNumber() const { return channel_number_; }
	const std::string getType() const { return type_; }
	const std::string getValue() const { return value_; }
	
};

class monitor {
private:
	std::string code_;
	int logger_channel_number_;
	std::vector<boost::shared_ptr<sensor> >  sensors_;
public:
	//Constructor
	monitor() : code_(""), logger_channel_number_(0) {}
	monitor(const std::string & code, int logger_channel_number):code_(code), logger_channel_number_(logger_channel_number) {}
	
	//Setters
	void setCode(const std::string & code) { code_ = code; }
	
	//Getters
	const std::string & getCode() const { return code_; }
	int getLoggerChannelNumber() const { return logger_channel_number_; }
	std::vector<boost::shared_ptr<sensor> > & getSensors() { return sensors_; }
};

class envi {
public:
	class data {
		private:
			time_t time_;
			std::string logger_code_;
			std::string xml_; 
			std::string error_;
			std::vector<boost::shared_ptr<monitor> > monitors_;
			bool valid_;
		public:
			data(time_t t=0, const std::string & str=""): time_(t), xml_(str) {}

			std::string xml() { return xml_; }
			void xml(std::string value) { xml_ = value; }
			void setLoggerCode(const std::string & value) { logger_code_ = value; }
			
			const std::string & getLoggerCode() { return logger_code_; }

			time_t time() { return time_; }
			void time(time_t value) { time_ = value; }
			void time(std::string value) { time_ = atol(value.c_str()); }
			void time(char * value) { time_ = atol(value); }
			bool parse();
			std::string error() { return error_; }	
			std::vector<boost::shared_ptr<monitor> > & getMonitors() { return monitors_; }
			
			std::string to_json();
	};
	
public:
	explicit envi(){}	
};
#endif // device_h__