/************************************************************************
	Copyright (C) 2010 Robert Scott.
	
	This file if part of envi_feed.
	
    envi_feed is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    envi_feed is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
	
*************************************************************************/

#include "config.h"
#include <string>
#include <vector>
#include <log4cplus/logger.h>
#include <tinyxml.h>


bool load_config(std::string filename, std::string & port, std::string &logger_code, std::vector<std::string> & endpoint_list)
{
	log4cplus::Logger logger=log4cplus::Logger::getRoot();
	std::ostringstream log_stream;
	TiXmlDocument doc(filename);
	doc.LoadFile();
	if(doc.Error()) {
		log_stream << "invalid config file(" << doc.ErrorDesc() <<") [" << filename << "]" ;
		LOG4CPLUS_ERROR(logger, log_stream.str());
		return false;	
	}
	TiXmlNode *root = doc.FirstChild("config");
	if(!root) {
		LOG4CPLUS_ERROR(logger,  "invalid config file (root element missing)");
		return false;	
	}

	TiXmlNode *logger_code_node = root->FirstChild("logger");
	if(logger_code_node) {
		logger_code = logger_code_node->FirstChild()->Value();
	}

	TiXmlNode *port_node = root->FirstChild("port");
	if(port_node) {
		port = port_node->FirstChild()->Value();
	}
	else {
		LOG4CPLUS_ERROR(logger,  "port not specified in config");
		return false;
	}
	
	TiXmlNode *ep = root->FirstChild("endpoint");
	while(ep) {
		endpoint_list.push_back(ep->FirstChild()->Value());
		ep = ep->NextSibling("endpoint");
	}
	
	int elements=endpoint_list.size();
	if(elements == 0) {
		LOG4CPLUS_ERROR(logger,  "endpoint not specified in config");
		return false;
	}
	return true;
}
