/************************************************************************
	Copyright (C) 2010 Robert Scott.
	
	This file if part of envi_feed.
	
    envi_feed is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    envi_feed is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
	
*************************************************************************/

#include "config.h"
#include <vector>
#include <string>
#include <boost/shared_ptr.hpp>
#include <log4cplus/logger.h>
#include <ctime>

#include "envi.h"

#ifndef _WIN32
extern sig_atomic_t run;
#else
extern int run;
#endif


void process_messages(boost::asio::serial_port * port, const std::string & logger_code, std::vector<std::string> & endpoints)
{
	boost::asio::streambuf stream_buffer;	
	log4cplus::Logger logger=log4cplus::Logger::getRoot();
	std::ostringstream log_message;
	zmq::context_t context(1);
	zmq::socket_t pub_socket(context,ZMQ_PUB);
	for(std::vector<std::string>::iterator it = endpoints.begin(); it!=endpoints.end();++it) {
		pub_socket.bind(it->c_str()); 
	}
	while(run) {
		std::string data;
		
		boost::asio::read_until(*port, stream_buffer, "\r\n");
		
		std::istream is(&stream_buffer);
		char cr, lf;
		is.unsetf(std::ios_base::skipws);
		is >> data >> cr >> lf;
		envi::data envi_data;
		envi_data.setLoggerCode(logger_code);
		envi_data.time(time(0));
		envi_data.xml(data);
		if(!envi_data.parse()) {
			continue;
		}

		std::ostringstream stream;
		stream << envi_data.to_json();
		std::string payload = stream.str();
		zmq::message_t pub_message(payload.length()+1);
		memcpy(pub_message.data(), payload.c_str(), payload.length()+1);
		pub_socket.send(pub_message);
		log_message.str("");
		log_message << "sent " << payload;
		LOG4CPLUS_DEBUG(logger, log_message.str());
	}
}